import 'package:rapidfire/Dynamic.dart';
import 'package:rapidfire/Api.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:sqflite/sqflite.dart';
import 'dart:io' as io;
import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';
import 'Annotation.dart' as annotation;
import 'Annotation.dart';
import 'Config.dart';
import 'GlobalScopeRF.dart';
import 'UtilRF.dart';
import 'package:rapidfire/GlobalRF.dart';
import 'package:flutter/material.dart';
import 'package:rapidfire/models/ModelDefaults.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'Const.dart';

class Db {
  static Database _database;

  Db();

  Future<Database> get database async {
    if (_database != null) return _database;
    _database = await initDatabase();
    return _database;
  }

  initDatabase() async {
    var pref = await SharedPreferences.getInstance();
    io.Directory documentDirectory = await getApplicationDocumentsDirectory();
    String path = join(documentDirectory.path, pref.getString(ConstRF.dbName));
    var db = await openDatabase(path, version: pref.getInt(ConstRF.dbVersion), onCreate: _onCreate);
    return db;
  }

  _onCreate(Database db, int version) {
    createTable().forEach((element) async {
      await db.execute(element);
      print(element);
    });
  }

  execute(String sql) async {
    var db = await database;
    var result = await db.rawUpdate(sql);
    return result;
  }

  Future<int> count(String sql) async {
    var db = await database;
    List<Map> result = await db.rawQuery(sql);
    return result.length;
  }

  get(Object model, String sql, [List<dynamic> values, Function onSuccess]) async {
    List<Object> modelList = List();
    var db = await database;
    List<Map> result = await db.rawQuery(sql, values);
    for (var row in result) {
      var newModel = Dynamic(model).newClass();
      for (var prop in row.entries) {
        Dynamic(newModel).set(prop.key, prop.value);
      }
      modelList.add(newModel);
    }
    if (onSuccess != null) onSuccess(modelList);
    return modelList;
  }

  getQuiz(Object model, String sql, [List<dynamic> values, Function onSuccess]) async {
    List<Object> modelList = List();
    var db = await database;
    List<Map> result = await db.rawQuery(sql, values);
    for (var row in result) {
      var newModel = Dynamic(model).newClass();
      for (var prop in row.entries) {
        Dynamic(newModel).set(prop.key, prop.value);
      }
      modelList.add(newModel);
    }
    if (onSuccess != null) onSuccess(modelList);
    return modelList;
  }

  get123(Object model, String sql, [List<dynamic> values, Function onSuccess]) async {
    List<Object> modelList = List();
    var db = await database;
    List<Map> result = await db.rawQuery(sql, values);
    result.forEach((element) {
      // remove forEach
      var newModel = Dynamic(model).newClass();

      element.forEach((key, value) {
        Dynamic(newModel).set(key, value);
      });
      modelList.add(newModel);
    });
    if (onSuccess != null) onSuccess(modelList);
    return modelList;
  }

  Future<Object> save(Object model) async {
    var pk = Dynamic(model).annotationData(annotation.Key);
    var value = Dynamic(model).get(pk.first['field']);

    var sql = 'select * from ${model.runtimeType.toString()} where ${pk.first['field']} = \'${Dynamic(model).get(pk.first['field'])}\'';
    List isExists = await Db().get(model, sql);

    if (value == null || value == "" || isExists.isEmpty)
      return await add(model, pk.first['field'], value);
    else
      return await update(model, '${pk.first['field']} = ?', ['$value']);
  }

  Future<Object> add(Object model, [String pkName, dynamic pkValue]) async {
    var db = await database;
    Dynamic dyn = Dynamic(model);

    if (dyn.hasBaseModel) await Libs.config?.model?.baseModelUpdater?.update(dyn);
    var data = dyn.modelToDataMap();
    if (pkValue == null || pkValue == "") data[pkName] = UtilRF.getUUID();
    var id = await db.insert(model.runtimeType.toString(), data);
    Dynamic(model).set(getPrimaryKeyName(model), data[pkName]);

    return model;
  }

  Future<Object> update(Object model, String where, List<dynamic> args) async {
    var db = await database;

    Dynamic dyn = Dynamic(model);

    if (dyn.hasBaseModel) await Libs.config?.model?.baseModelUpdater?.update(dyn);
    var data = Dynamic(model).modelToDataMap();
    await db.update(model.runtimeType.toString(), data, where: where, whereArgs: args);
    return model;
  }

  Future<int> delete(Object model, String id, [List<Function> validators, bool isChildDelete = true]) async {
    validators = validators ?? Libs.config.model.validators;
    var isValidValidForDeletion = true;
    var db = await database;
    var pk = Dynamic(model).annotationData(annotation.Key);
    validators?.forEach((element) {
      isValidValidForDeletion = element(null);
      if (!isValidValidForDeletion) return;
    });
    if (isValidValidForDeletion) {
      var result;
      result = await db.delete(
        model.runtimeType.toString(),
        where: '${pk.first['field']} = ?',
        whereArgs: [id],
      );
      if (isChildDelete) {
        for (var item in Dynamic(model).childModels) {
          result = await db.delete(
            item,
            where: '${pk.first['field']} = ?',
            whereArgs: [id],
          );
        }
      }
    } else
      return Future.value(0);
  }

  Future close() async {
    var db = await database;
    db.close();
  }

  List<String> createTable() {
    List<String> createSql = new List();
    var pkSql = "";
    var pkFieldName = "";
    var pkFieldType = "";
    var pkFieldAuto = "";
    var models = Libs.appModels.get(); // New Model Style

    models.forEach((k, v) {
      var _annotation = Dynamic((v as ModelRF).model).annotationData(annotation.Key).first;

      if (_annotation != null) {
        pkFieldName = _annotation["field"];
        pkFieldType = _annotation["type"].toString();
        pkFieldAuto = _annotation["data"]["auto"] ? "AUTOINCREMENT" : "";
        pkSql = "${pkFieldName} ${convertSqlFieldType(pkFieldType)} PRIMARY KEY ${pkFieldAuto}";
      }

      var sql = "";
      Dynamic((v as ModelRF).model).modelMembers.forEach((name, type) {
        if (pkFieldName != name) {
          if (!type.toString().contains('List')) {
            sql = sql + name + " " + convertSqlFieldType(type.toString()) + ", ";
          }
        }
      });
      sql = sql.trim().substring(0, sql.length - 2);
      createSql.add("CREATE TABLE ${k}(${pkSql},${sql});");
    });
    return createSql;
  }

  String convertSqlFieldType(String s) {
    String type = "";
    if (s.contains("String")) {
      type = " TEXT";
    } else if (s.contains("long")) {
      type = " INTEGER";
    } else if (s.contains("byte[]")) {
      type = " BLOB";
    } else if (s.contains("int")) {
      type = " INTEGER";
    } else {
      type = "";
    }
    return type;
  }

  getPrimaryKeyName(Object model) {
    var pk = Dynamic(model).annotationData(annotation.Key);
    return pk.length == 0 ? null : pk.first['field'];
  }

  getPrimaryKeyValue(Object model) {
    var _model = Dynamic(model);
    var pk = _model.annotationData(annotation.Key);
    var pkName = pk.first['field'];
    var pkValue = _model.get(pkName.toString());
    return pkValue;
  }

  static getModelDefaults(String modelName, String fieldName) {
    return getDropDownItems(ModelDefault(), "OptionValue", "OptionText", where: 'where ModelName = \'$modelName\' and FieldName = \'$fieldName\'');
  }

  static Future<List<DropdownMenuItem<String>>> getDropDownItems(
    Object model,
    String valueText,
    String displayText, {
    String where = '',
    String orderSort = 'ASC',
    String orderBy = '1',
    String defaults = 'Select an Item',
    bool defaultOption = true,
  }) async {
    var dropdownItems = List<DropdownMenuItem<String>>();
    var sql = 'SELECT $valueText, $displayText FROM ${model.runtimeType.toString()} $where Order By $orderBy $orderSort';
    print(sql);
    var modelList = await rf.db.get(model, sql);
    if (defaultOption && defaults.isNotEmpty) dropdownItems.add(DropdownMenuItem(child: Text(defaults), value: "0"));

    if (modelList != null) {
      for (var item in modelList) {
        var value = Dynamic(item).get(valueText);
        var display = Dynamic(item).get(displayText) ?? '';
        dropdownItems.add(
          DropdownMenuItem(child: Text(display), value: value.toString()),
        );
      }
    }

    return dropdownItems;
  }

  static Future<List<RadioCheckData>> getCheklistItems(
    Object model,
    String valueText,
    String displayText, {
    String where = '',
    String orderSort = 'ASC',
    String orderBy = '1',
  }) async {
    var checkItems = List<RadioCheckData>();
    var sql = 'SELECT $valueText, $displayText FROM ${model.runtimeType.toString()} $where Order By $orderBy $orderSort';
    print(sql);
    var modelList = await rf.db.get(model, sql);

    if (modelList != null) {
      for (var item in modelList) {
        var value = Dynamic(item).get(valueText);
        var display = Dynamic(item).get(displayText) ?? '';
        checkItems.add(
          RadioCheckData(label: display.toString(), value: value.toString()),
        );
      }
    }

    return checkItems;
  }

  static getDropdownText(String modelName, String fieldName, dynamic value) {
    return getDropDownItemText(ModelDefault(), "OptionValue", "OptionText", where: 'where ModelName = \'$modelName\' and FieldName = \'$fieldName\' and OptionValue = $value');
  }

  static Future<String> getDropDownItemText(
    Object model,
    String valueText,
    String displayText, {
    String where = '',
    String order = 'ASC',
    String defaults = 'Select an Item',
  }) async {
    String text = "";
    var sql = 'SELECT $valueText, $displayText FROM ${model.runtimeType.toString()} $where Order By 1 $order';
    print(sql);
    var modelList = await rf.db.get(model, sql);
    if (modelList != null) {
      for (var item in modelList) {
        var value = Dynamic(item).get(valueText);
        var display = Dynamic(item).get(displayText) ?? '';
        text = display.toString();
      }
    }
    return text;
  }
}

enum DataSourceType { local, remote, memory }
enum DataSourceReadType { auto, none, quick, advanced }

class DataSourceRF<Object> {
  DataSourceType type;
  List<Object> modelList;
  Map<String, dynamic> mapMultiCheck = Map();

  int currentPage = 1;
  String sql = '';
  String customSql = '';
  String externalWhereExp = '';
  bool isLoading = false;
  Function onFilter;
  Function updateState;

  String selectFields = '*';
  String whereFields = '';
  String orderField;
  String orderSort;
  String gropBy;

  var currentReadType;
  dynamic model;
  Function stateUpdateFn;
  Function readListItemFn;
  Function deleteListItemFn;
  Function updateLoadingStateFn;

  DataSourceRF({
    this.modelList,
    this.type = DataSourceType.local,
    this.externalWhereExp = '',
    this.customSql = '',
    this.orderField = 'DataCollectionDate',
    this.orderSort = 'desc',
    this.gropBy = '',
  });

  //#region NewDataSource
  updateSql(int pageNo) {
    if (!orderField.toLowerCase().contains('date(') && orderField.toLowerCase().contains('date')) {
      orderField = 'date($orderField)';
    } else if (orderField.toLowerCase().contains('Sendattime')) {
      orderField = 'date($orderField)';
    }

    this.model = Dynamic.genericModelName(modelList);
    var ps = Libs.config.data.pageSize;
    var extWhere = externalWhereExp.isNotEmpty ? (whereFields.isEmpty ? ' where $externalWhereExp' : ' and $externalWhereExp') : '';
    var sql = 'select $selectFields from ${this.model} ${whereFields.isEmpty ? '' : ' where '}'
        ' ${whereFields} $extWhere ${gropBy} '
        'order by $orderField $orderSort limit $ps offset ${ps * pageNo}';
    if (this.customSql.isNotEmpty) {
      this.sql = '$customSql limit $ps offset ${ps * pageNo}';
    }
    print('ext: $sql');
    print(sql);
    this.sql = sql;
  }

  read(int pageNumber, int pageSize, dynamic listView) async {
    var models;

    if (this.type == DataSourceType.local) {
      //if (pageNumber == 0) await Future.delayed(Duration(seconds: 1));
      print('on pull page : ${pageNumber}');
      print('on pull data : ${this.sql}');
      Models modelList = new Models();
      for (var entry in modelList.get().entries) {
        print(entry.key.toString() + " : " + this.model.toString());
        if (entry.key == this.model.toString()) {
          ModelRF m = entry.value as ModelRF;
          models = await Db().get(m.model, this.sql);

          break;
        }
      }
      if (models.length == 0) {
        currentPage--; //currentPage--;
      }
      if (this.updateState != null) this.updateState(models);
    } else if (this.type == DataSourceType.remote) {
    } else if (this.type == DataSourceType.memory) {}

    //if (pageNumber == 0) modelList.clear();
    /*if (listView.mounted) {
      listView.setState(() {
        if (this.type == DataSourceType.local) {
          if (models != null) for (var item in models) modelList.add(item);
        }
        isLoading = false;
      });
    }*/
    //if (this.type == DataSourceType.memory) {}
  }

  Future<int> getCount({bool withExternal = true}) async {
    String query = '';
    withExternal
        ? query = (this.externalWhereExp.isEmpty
            ? 'SELECT * FROM ${Dynamic.genericModelName(modelList)} ${gropBy}'
            : 'SELECT * FROM ${Dynamic.genericModelName(modelList)} where ${this.externalWhereExp} ${gropBy}')
        : query = 'SELECT * FROM ${Dynamic.genericModelName(modelList)} ${gropBy}';

    int count = await rf.db.count(query);
    return count;
  }

  loadData() async {
    var models;
    this.model = Dynamic.genericModelName(modelList);
    if (this.type == DataSourceType.local) {
      Models modelsList = new Models();
      for (var entry in modelsList.get().entries) {
        if (entry.key == this.model.toString()) {
          ModelRF m = entry.value as ModelRF;
          String query = this.externalWhereExp.isEmpty ? 'SELECT * FROM ${this.model.toString()}' : 'SELECT * FROM ${this.model.toString()} where ${this.externalWhereExp}';
          models = await Db().get(m.model, query);
          break;
        }
      }
      if (models != null && models.length > 0) {
        for (var item in models) modelList.add(item);
      }
    }
  }

  filterQuick(sqlOrExp) {
    modelList.clear();
    var instructions = sqlOrExp.split('#');
    var _whereFields = instructions[0];
    var queryValue = instructions[1];
    var sortType = instructions[2];
    this.whereFields = queryValue.isEmpty ? '' : quickFilterWhereOr(_whereFields, queryValue);
    updateSql(0);
    if (onFilter != null) onFilter();
  }

  quickFilterWhereOr(String clause, value) {
    String where = "";
    List<String> clauses = clause.split(',');
    clauses.forEach((element) {
      where += ' $element like "%$value%" or ';
    });
    where = where.substring(0, where.length - 3);
    return where;
  }

  filter([String sql]) async {
    modelList.clear();

    this.currentPage = 1;
    this.whereFields = sql ?? '';
    updateSql(0);
    if (onFilter != null) await onFilter();
  }

  filterClear() async {
    modelList.clear();
    this.currentPage = 1;
    this.whereFields = '';
    updateSql(0);
    if (onFilter != null) await onFilter();
  }

  refresh() async {
    await filterClear();
    mapMultiCheck.clear();
  }

  Future<Null> reload(BuildContext context) async {
    if (!PageRF.keyboardOpened && PageRF.pageViewInsets == 0) {
      await filterClear();
      mapMultiCheck.clear();
      PageRF.keyboardOpened = true;
    } else {
      PageRF.keyboardOpened = true;
    }
  }

  //#endregion NewDataSource

  clear() {
    //???
    modelList.clear();
    mapMultiCheck.clear();
  }

  get items {
    return modelList;
  }

  add(dynamic data) {
    if (data is List) {
      for (var item in data) {
        modelList.add(item);
      }
    } else {
      modelList.add(data);
    }
  }

  update(dynamic model, int index) {
    modelList[index] = model;
  }

  delete(int index) {
    //????
    if (deleteListItemFn != null) deleteListItemFn(index);
  }

  void save(dynamic parentModel) async {
    if (parentModel == null) return;

    var fpk = Dynamic(parentModel).annotationData(annotation.Key);
    var fpkName = fpk.first['field'];
    var fpkValue = Dynamic(parentModel).get(fpkName);

    if (modelList.length > 0) model = modelList.first.runtimeType;

    await rf.db.execute('delete from ${model.toString()} where $fpkName = \'$fpkValue\'');

    var _model = (model.runtimeType == String) ? Models.models[model].model : model;
    var pk = Dynamic(_model).annotationData(annotation.Key);
    var pkName = pk.first['field'];
    for (var item in modelList) {
      Dynamic(item).set(pkName, '');
      Dynamic(item).set(fpkName, fpkValue);
      await rf.db.save(item);
    }
  }

  void updateSelectStatus(bool checkValue, dynamic sourceModel, dynamic destinationModel, List<dynamic> fields) {
    var dynSourcePK = Db().getPrimaryKeyValue(sourceModel);

    //var destinationModel = this.model;

    var _destinationModel = Dynamic(destinationModel).newClass();
    fields.forEach((fieldName) {
      Dynamic(_destinationModel).set(fieldName, Dynamic(sourceModel).get(fieldName));
    });

    Dynamic(sourceModel).set('IsActive', checkValue ? 0 : 1);

    if (checkValue) {
      mapMultiCheck[dynSourcePK] = _destinationModel;
    } else {
      mapMultiCheck.remove(dynSourcePK);
    }

    //modelList.clear();
    mapMultiCheck.forEach((key, value) {
      modelList..removeWhere((element) => Dynamic(element).get(fields.first.toString()) == Dynamic(value).get(fields.first.toString()));
      modelList.add(value);
    });

    if (onFilter != null) onFilter();
    DialogRF.refresh();
  }

// readOLD(String sqlParam, int pageNo, [DataSourceReadType readType = DataSourceReadType.none]) async {
//   if (!isLoading) {
//     isLoading = true;
//     if (updateLoadingStateFn != null) updateLoadingStateFn(true);
//     var data;
//     Api api = Api();
//
//     var model = typeOfElementsInList(modelList);
//
//     processWhereOr(String clause, value) {
//       String where = "";
//       List<String> clauses = clause.split(',');
//       clauses.forEach((element) {
//         where += ' $element like "%$value%" or ';
//       });
//       where = where.substring(0, where.length - 3);
//       return where;
//     }
//
//     if (readType == DataSourceReadType.none) {
//       currentReadType = DataSourceReadType.none;
//       updateSql(pageNo, 'desc');
//     } else if (readType == DataSourceReadType.auto) {
//       currentReadType = DataSourceReadType.auto;
//       updateSql(pageNo, 'desc');
//     } else if (readType == DataSourceReadType.quick) {
//       currentReadType = DataSourceReadType.quick;
//       var instructions = sqlParam.split('#');
//       var modelName = this.model;
//       var _whereFields = instructions[0];
//       var queryValue = instructions[1];
//       var sortType = instructions[2];
//       this.whereFields = queryValue.isEmpty ? '' : processWhereOr(_whereFields, queryValue);
//       updateSql(pageNo, 'desc');
//     } else if (readType == DataSourceReadType.advanced) {
//       currentReadType = DataSourceReadType.advanced;
//       this.whereFields = sqlParam;
//       updateSql(pageNo, 'desc');
//     }
//
//     if (type == DataSourceType.local) {
//       data = await Db().get(model, this.sql);
//       if (data.length == 0) {
//         if (currentPage > 1) currentPage--;
//         pageNo = currentPage;
//       }
//     } else if (type == DataSourceType.remote) {
//       //data = await api.fetchRND(model, pageNo: pageNo);
//       await api.remoteGet(model, pageNo, (_, __, rec) {
//         data = rec;
//       });
//     } else if (type == DataSourceType.memory) {
//       data = modelList;
//     }
//
//     if (stateUpdateFn != null) stateUpdateFn(data, pageNo);
//
//     // await Future.delayed(Duration(microseconds: 5000));
//     print('from call:false');
//     if (updateLoadingStateFn != null) updateLoadingStateFn(false);
//     isLoading = false;
//   }
// }

// refreshOLD() {
//   if (currentReadType == DataSourceReadType.none) {
//     readOLD(sql, 0, DataSourceReadType.none);
//   }
// }
}
