import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:rapidfire/RapidFire.dart';
import 'package:rapidfire/chatting/ChatGroupsPage.dart';
import 'package:rapidfire/models/ChatModels.dart';

class ChatPage extends StatefulWidget {
  String chatGroupId;
  ChatGroup chatGroup;
  String userId;
  Color primaryColor;
  Color secondaryColor;
  Color chatHeadColor;
  double bubbleRadious;
  bool showArrow;
  bool fromGroupPage;
  bool hasModifyPermission;

  @override
  _ChatPageState createState() => _ChatPageState();

  ChatPage(this.chatGroupId, this.userId, {this.hasModifyPermission = true, this.primaryColor, this.secondaryColor, this.bubbleRadious = 50, this.chatHeadColor, this.showArrow, this.fromGroupPage});
}

class _ChatPageState extends State<ChatPage> {
  var messageCtrl = TextEditingController();
  ScrollController _scrollController = new ScrollController();
  var listener;
  int mCounter = -1;
  int notificationcount = 0;
  double actualHeight = 100;
  double maxBubbleWidth = 80.0;
  List<ChatMessage> messageList = [];
  List<ChatGroupMember> chatMembers = [];
  bool isUpdating = true;
  Map<String, String> members = {};
  Map<String, Color> memberColor = {};
  List<String> seens = [];

  List<Color> colors = [
    Color(0xff71b4fb),
    Color(0xfffa8c73),
    Color(0xff8873f4),
    Color(0xff02c39a),
    Color(0xff70a288),
    Color(0xfff15bb5),
    Color(0xffedb183),
    Color(0xffa5668b),
    Color(0xff8873f4),
  ];

  @override
  void initState() {
    super.initState();
    listener = FirebaseDatabase.instance.reference().child('messages/${widget.chatGroupId}').onValue.listen((event) {
      if (notificationcount > 0) {
        print('message listener called:' + notificationcount.toString());
        bool isNotExist = messageList.where((element) => element.ChatMessageId == event.snapshot.value['ChatMessageId']).isEmpty;

        var data = event.snapshot;
        if (isNotExist) {
          ChatMessage rdbMessage = ChatMessage(
            ChatMessageId: data.value['ChatMessageId'],
            Message: data.value['Message'],
            SendAt: data.value['SendAt'],
            SendTo: data.value['SendTo'],
            Sender: data.value['Sender'],
            SendAtText: data.value['SendAtText'],
            Type: data.value['Type'],
            ChatSeenUser: [],
          );
          Map<dynamic, dynamic> result = data.value['SeenUsers'];
          if (result != null) {
            result.forEach((key, value) {
              print('seenUser: ' + value['UserId']);
              seens.remove(value['UserId']);
              rdbMessage.ChatSeenUser.add(value['UserId']);
            });
          }
          messageList.insert(0, rdbMessage);
          actualHeight = (messageList.length * 70).toDouble();
          notificationcount = 0;
          setState(() {});
          rf.chat.setSeen(widget.chatGroupId, widget.userId, rdbMessage.ChatMessageId);
        } else {
          ChatMessage chatMsg = messageList.where((element) => element.ChatMessageId == event.snapshot.value['ChatMessageId']).last;
          Map<dynamic, dynamic> result = data.value['SeenUsers'];
          if (result != null) {
            result.forEach((key, value) {
              print('seenUser: ' + value['UserId']);
              if (chatMsg.ChatSeenUser != null && !chatMsg.ChatSeenUser.contains(value['UserId'])) {
                seens.remove(value['UserId']);
                chatMsg.ChatSeenUser.add(value['UserId']);
              }
            });
            setState(() {});
          }
        }
      }
      notificationcount++;
    });
    _scrollController.addListener(_scrollListener);
    initCall();
  }

  @override
  void dispose() {
    super.dispose();
    _scrollController.dispose();
    if (listener != null) listener.cancel();
  }

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();
  }

  initCall() async {
    print('init Call');
    var data = await rf.chat.getChatGroup(widget.chatGroupId, widget.userId);
    if (data != null) {
      widget.chatGroup = data;
      setState(() {});
      var chatGroupMembers = widget.chatGroup.ChatGroupMember;
      chatMembers.clear();
      for (ChatGroupMember cm in chatGroupMembers) {
        chatMembers.add(cm);
        if (!memberColor.containsKey(cm.UserId)) {
          if (mCounter == -1) {
            memberColor[cm.UserId] = widget.chatHeadColor;
            mCounter++;
          } else {
            memberColor[cm.UserId] = colors[mCounter];
            mCounter++;
          }
          if (mCounter == 9) mCounter = 0;
        }
      }
      print('Members:' + chatMembers.length.toString());
      updateMessages();
    }
  }

  @override
  Widget build(BuildContext context) {
    print('CHATPAGE BUILD');
    maxBubbleWidth = (ScreenRF.width(context) - 80).toDouble();
    seens = [];
    return WillPopScope(
      onWillPop: () {
        if (widget.fromGroupPage)
          rf.chat.callChatgroupPage(replace: true);
        else
          rf.page.back(context);
      },
      child: Scaffold(
        appBar: AppBar(
          automaticallyImplyLeading: false,
          backgroundColor: Colors.grey[100],
          title: Container(
            child: Row(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                InkWell(
                  onTap: () {
                    if (widget.fromGroupPage)
                      rf.chat.callChatgroupPage(replace: true);
                    else
                      rf.page.back(context);
                  }, //rf.page.back(_context),
                  child: Container(margin: const EdgeInsets.all(5), child: Icon(Icons.arrow_back_ios, color: Colors.black)),
                ),
                /*Container(
                  height: 40,
                  width: 40,
                  margin: const EdgeInsets.all(5),
                  decoration: BoxDecoration(
                    color: widget.chatHeadColor ?? Colors.deepPurple,
                    borderRadius: BorderRadius.all(Radius.circular(50)),
                    boxShadow: <BoxShadow>[
                      BoxShadow(
                        offset: Offset(0, 0),
                        blurRadius: 5,
                        color: widget.chatHeadColor.withOpacity(0.5) ?? Colors.deepPurple.withOpacity(.5),
                      )
                    ],
                  ),
                  child: Center(
                      child: TextRF(
                    UtilRF.getInitials(widget.chatGroup.Name),
                    size: 20,
                    color: Colors.white,
                    weight: FontWeight.bold,
                  )),
                ),*/
                SizedBox(
                  width: 10,
                ),
                Container(
                  width: 280,
                  child: TextRF(
                    widget.chatGroup == null ? '' : widget.chatGroup.Name,
                    size: 18,
                    overflow: TextOverflow.ellipsis,
                    color: Colors.black,
                    weight: FontWeight.bold,
                  ),
                ),
              ],
            ),
          ),
          actions: [
            InkWell(
              onTap: () {
                chatMembersDialog();
              },
              child: Icon(
                Icons.workspaces_outline,
                color: Colors.red,
              ),
            ),
            SizedBox(
              width: 10,
            )
          ],
        ),
        body: widget.chatGroup == null
            ? Container()
            : SingleChildScrollView(
                child: SafeArea(
                  child: Container(
                    height: ScreenRF.height(context) - 80,
                    color: Colors.white,
                    child: Column(
                      children: [
                        Expanded(
                          flex: 1,
                          child: SizedBox(),
                        ),
                        Container(
                          height: actualHeight > (ScreenRF.height(context) - 160) ? (ScreenRF.height(context) - 160) : actualHeight,
                          width: MediaQuery.of(context).size.width,
                          child: ListView.builder(
                            controller: _scrollController,
                            reverse: true,
                            shrinkWrap: true,
                            itemCount: messageList.length,
                            itemBuilder: (context, index) {
                              ChatMessage chatMsg = messageList[index];
                              return Container(
                                  width: MediaQuery.of(context).size.width - 100,
                                  margin: const EdgeInsets.only(bottom: 10),
                                  decoration: BoxDecoration(
                                    color: Colors.white,
                                    borderRadius: BorderRadius.all(Radius.circular(5)),
                                  ),
                                  child: chatMsg.Type == 'notification'
                                      ? notificationCard(chatMsg)
                                      : widget.userId == chatMsg.Sender
                                          ? myCard(chatMsg)
                                          : othersCard(chatMsg));
                            },
                          ),
                        ),
                        Container(
                          height: 80,
                          width: ScreenRF.width(context),
                          padding: const EdgeInsets.all(5),
                          decoration: BoxDecoration(
                            color: Colors.grey.withOpacity(0.2),
                            borderRadius: BorderRadius.all(Radius.circular(0)),
                          ),
                          child: Row(
                            mainAxisSize: MainAxisSize.max,
                            children: [
                              Container(
                                height: 45,
                                width: ScreenRF.width(context) - 100,
                                margin: const EdgeInsets.all(8),
                                padding: const EdgeInsets.all(5),
                                decoration: BoxDecoration(
                                  color: Colors.white,
                                  borderRadius: BorderRadius.all(Radius.circular(50)),
                                ),
                                child: Center(
                                  child: TextField(
                                    controller: messageCtrl,
                                    style: TextStyle(fontSize: 18, color: Colors.black),
                                    enabled: true,
                                    keyboardType: TextInputType.text,
                                    //autofocus: false,
                                    decoration: new InputDecoration(
                                      fillColor: Colors.transparent,
                                      border: InputBorder.none,
                                      focusColor: Colors.transparent,
                                      filled: true,
                                      hintText: "Write your message",
                                      hintStyle: TextStyle(color: Colors.grey[300], fontWeight: FontWeight.w300),
                                    ),
                                  ),
                                ),
                              ),
                              Expanded(
                                child: SizedBox(),
                              ),
                              Container(
                                child: InkWell(
                                  onTap: () {
                                    if (messageCtrl.text.isNotEmpty) {
                                      FocusScope.of(context).unfocus();
                                      rf.chat.sendMessage(widget.chatGroup.ChatGroupId, widget.userId, messageCtrl.text);
                                      messageCtrl.clear();
                                      setState(() {});
                                    }
                                  },
                                  child: Icon(
                                    Icons.send,
                                    color: Colors.grey,
                                  ),
                                ),
                              ),
                              Expanded(
                                child: SizedBox(),
                              ),
                            ],
                          ),
                        ),
                      ],
                    ),
                  ),
                ),
              ),
      ),
    );
  }

  //region List Cards

  notificationCard(ChatMessage chatMessage) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Container(
          margin: const EdgeInsets.all(6),
          padding: const EdgeInsets.only(left: 10, right: 10, top: 5, bottom: 5),
          height: 30,
          decoration: BoxDecoration(
            color: Colors.grey.withOpacity(0.1),
            borderRadius: BorderRadius.all(Radius.circular(30)),
          ),
          child: Center(
            child: TextRF(
              chatMessage.Message,
              size: 12,
              weight: FontWeight.w400,
              color: Colors.grey,
            ),
          ),
        ),
      ],
    );
  }

  myCard(ChatMessage chatMessage) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.end,
      children: [
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: [
            TextRF(
              chatMessage.SendAtText,
              color: Colors.grey,
              size: 10,
              weight: FontWeight.w300,
            ),
            Container(
              constraints: BoxConstraints(minWidth: 10, maxWidth: maxBubbleWidth),
              padding: EdgeInsets.all(widget.bubbleRadious / 4),
              margin: const EdgeInsets.only(top: 6, left: 6, right: 6, bottom: 2),
              decoration: BoxDecoration(
                color: widget.primaryColor ?? Colors.blue.withOpacity(0.7),
                borderRadius: BorderRadius.only(
                    topRight: Radius.circular(3),
                    bottomLeft: Radius.circular(widget.bubbleRadious ?? 50),
                    topLeft: Radius.circular(widget.bubbleRadious ?? 50),
                    bottomRight: Radius.circular(widget.bubbleRadious ?? 50)),
              ),
              child: TextRF(
                chatMessage.Message,
                color: Colors.white,
              ),
            ),
          ],
        ),
        (chatMessage.ChatSeenUser != null && chatMessage.ChatSeenUser.length > 0)
            ? FutureBuilder(
                future: getSeenUsers(chatMessage.ChatSeenUser, true),
                builder: (BuildContext context, AsyncSnapshot<dynamic> snapshot) {
                  if (snapshot.hasData) {
                    return snapshot.data;
                  } else {
                    return Text("");
                  }
                },
              )
            : Container(),
      ],
    );
  }

  othersCard(ChatMessage chatMessage) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        Row(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.end,
          children: [
            Container(
              height: 30,
              width: 30,
              margin: const EdgeInsets.all(5),
              decoration: BoxDecoration(
                color: memberColor[chatMessage.Sender] ?? widget.chatHeadColor,
                borderRadius: BorderRadius.all(Radius.circular(50)),
                boxShadow: <BoxShadow>[
                  BoxShadow(
                    offset: Offset(0, 0),
                    blurRadius: 2,
                    color: memberColor[chatMessage.Sender] ?? widget.chatHeadColor.withOpacity(0.5),
                  )
                ],
              ),
              child: Center(
                child: FutureBuilder(
                  future: getUserName(chatMessage.Sender),
                  builder: (BuildContext context, AsyncSnapshot<String> snapshot) {
                    if (snapshot.hasData) {
                      return TextRF(
                        UtilRF.getInitials(snapshot.data) ?? '0',
                        size: 16,
                        color: Colors.white,
                        weight: FontWeight.bold,
                      );
                    } else {
                      return Text("");
                    }
                  },
                ),
              ),
            ),
            Container(
              constraints: BoxConstraints(minWidth: 10, maxWidth: maxBubbleWidth - 30),
              padding: EdgeInsets.all(widget.bubbleRadious / 4),
              margin: const EdgeInsets.only(left: 0, right: 6, bottom: 5, top: 6),
              decoration: BoxDecoration(
                color: widget.secondaryColor ?? Colors.grey.withOpacity(0.25),
                borderRadius: BorderRadius.only(
                    bottomLeft: Radius.circular(3),
                    topRight: Radius.circular(widget.bubbleRadious ?? 50),
                    topLeft: Radius.circular(widget.bubbleRadious ?? 50),
                    bottomRight: Radius.circular(widget.bubbleRadious ?? 50)),
              ),
              child: TextRF(
                chatMessage.Message,
                color: Colors.black,
              ),
            ),
            TextRF(
              chatMessage.SendAtText,
              color: Colors.grey,
              size: 10,
              weight: FontWeight.w300,
            ),
          ],
        ),
        (chatMessage.ChatSeenUser != null && chatMessage.ChatSeenUser.length > 0)
            ? FutureBuilder(
                future: getSeenUsers(chatMessage.ChatSeenUser, false),
                builder: (BuildContext context, AsyncSnapshot<dynamic> snapshot) {
                  if (snapshot.hasData) {
                    return snapshot.data;
                  } else {
                    return Text("");
                  }
                },
              )
            : Container(),
      ],
    );
  }

  memberCard({String userId, String name, bool isAdd = false, bool showButton = true, String userTable, Function onDelete, Function onAdded}) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        Row(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Container(
              height: 30,
              width: 30,
              margin: const EdgeInsets.all(5),
              decoration: BoxDecoration(
                color: memberColor[userId] ?? widget.chatHeadColor,
                borderRadius: BorderRadius.all(Radius.circular(50)),
                boxShadow: <BoxShadow>[
                  BoxShadow(
                    offset: Offset(0, 0),
                    blurRadius: 2,
                    color: memberColor[userId] ?? widget.chatHeadColor.withOpacity(0.5),
                  )
                ],
              ),
              child: Center(
                child: TextRF(
                  UtilRF.getInitials(name) ?? '0',
                  size: 16,
                  color: Colors.white,
                  weight: FontWeight.bold,
                ),
              ),
            ),
            Container(
              padding: const EdgeInsets.all(6),
              margin: const EdgeInsets.all(5),
              decoration: BoxDecoration(
                color: widget.secondaryColor ?? Colors.grey.withOpacity(0.25),
                borderRadius: BorderRadius.all(Radius.circular(10)),
              ),
              child: TextRF(
                name,
                color: Colors.black,
              ),
            ),
            (widget.hasModifyPermission && showButton)
                ? Container(
                    child: Center(
                      child: InkWell(
                        onTap: () async {
                          if (isAdd) {
                            await rf.chat.addGroupMember(chatGroupId: widget.chatGroup.ChatGroupId, userId: userId, userTable: userTable);
                            onAdded.call();
                          } else
                            DialogRF(context,
                                height: 110,
                                avatarIcon: FontAwesomeIcons.save,
                                hideAvatar: true,
                                avatarIconSize: 25,
                                hideDefaultControls: true,
                                type: DialogType.popup,
                                title: 'Remove Member',
                                subTitle: 'confirmation',
                                body: Column(
                                  children: [
                                    Center(
                                        child: TextRF(
                                      'Do you want to remove below member from this group ?',
                                      color: Colors.black,
                                      size: 18,
                                      weight: FontWeight.w400,
                                    )),
                                    SizedBox(
                                      height: 10,
                                    ),
                                    Center(
                                        child: TextRF(
                                      name,
                                      color: Colors.red,
                                      size: 20,
                                      weight: FontWeight.w500,
                                    )),
                                  ],
                                ),
                                onAccept: () async {
                                  await rf.chat.removeGroupMember(userId: userId, chatGroupId: widget.chatGroup.ChatGroupId, userTable: userTable).then((value) {
                                    initCall();
                                    onDelete.call();
                                  });
                                },
                                onCancel: () => rf.page.back(rf.context));
                        },
                        child: Icon(
                          isAdd ? Icons.add_box_outlined : Icons.delete_rounded,
                          color: isAdd ? Colors.green : Colors.red,
                          size: 20,
                        ),
                      ),
                    ),
                  )
                : Container(),
          ],
        ),
      ],
    );
  }

  //endregion

  updateMessages({String sendAt = ''}) async {
    rf.chat.getMessages(widget.chatGroup.ChatGroupId, sendAt: sendAt, userId: widget.userId).then((value) {
      print('update Message');
      if (value.length > 0) {
        if (sendAt.isEmpty) {
          messageList = value;
          actualHeight = (messageList.length * 60).toDouble();
          print(messageList.first.Message);
          rf.chat.setSeen(widget.chatGroup.ChatGroupId, widget.userId, messageList.first.ChatMessageId);
          setState(() {});
        } else {
          messageList.addAll(value);
          setState(() {});
        }
        isUpdating = false;
      }
    });
  }

  Future<String> getUserName(String userid) async {
    print('userId:' + userid);
    if (members.containsKey(userid)) {
      print('userName :' + members[userid]);
      return members[userid];
    } else {
      DataSnapshot datas = await FirebaseDatabase.instance.reference().get();
      DataSnapshot data = await FirebaseDatabase.instance.reference().child('users/${userid}').get();
      String name = data.value['UserName'];
      print('Id of the User: $userid');
      print('Name of the User: $name');
      members[userid] = name;
      if (!memberColor.containsKey(userid)) {
        if (mCounter == -1) {
          memberColor[userid] = widget.chatHeadColor;
          mCounter++;
        } else {
          memberColor[userid] = colors[mCounter];
          mCounter++;
        }
        if (mCounter == 9) mCounter = 0;
      }
      return name;
    }
  }

  Future<dynamic> getSeenUsers(List<String> data, bool isSelf) async {
    var mRow = Row(
      mainAxisAlignment: isSelf ? MainAxisAlignment.end : MainAxisAlignment.start,
      children: [],
    );
    if (!isSelf) {
      mRow.children.add(SizedBox(
        width: 40,
      ));
    } else {
      mRow.children.add(Expanded(
        child: SizedBox(),
      ));
    }
    for (String ds in data) {
      if (ds != widget.userId && !seens.contains(ds)) {
        seens.add(ds);
        var nData = Container(
          height: 15,
          width: 15,
          decoration: BoxDecoration(
            color: memberColor[ds] ?? widget.chatHeadColor,
            borderRadius: BorderRadius.all(Radius.circular(50)),
          ),
          child: Center(
            child: FutureBuilder(
              future: getUserName(ds),
              builder: (BuildContext context, AsyncSnapshot<String> snapshot) {
                if (snapshot.hasData) {
                  return TextRF(
                    UtilRF.getInitials(snapshot.data) ?? '0',
                    size: 9,
                    color: Colors.white,
                    weight: FontWeight.normal,
                  );
                } else {
                  return Text("");
                }
              },
            ),
          ),
        );
        mRow.children.add(nData);
      }
    }
    mRow.children.add(SizedBox(
      width: 20,
    ));
    return Container(width: maxBubbleWidth, child: mRow);
  }

  chatMembersDialog() {
    bool isMemberAdd = false;
    var searchCTRL = TextEditingController();
    String searchText;
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      barrierColor: Colors.black.withOpacity(0.8),
      backgroundColor: Colors.transparent,
      builder: (BuildContext context) {
        return StatefulBuilder(
          builder: (BuildContext context, setState) {
            return Container(
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.only(topRight: Radius.circular(20), topLeft: Radius.circular(20)),
              ),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  SizedBox(
                    height: 20,
                  ),
                  Row(
                    children: [
                      Expanded(
                        child: SizedBox(),
                      ),
                      Container(
                        width: 80,
                        height: 80,
                        decoration: BoxDecoration(
                          color: widget.chatHeadColor ?? Colors.brown,
                          borderRadius: BorderRadius.all(Radius.circular(50)),
                          boxShadow: <BoxShadow>[
                            BoxShadow(
                              offset: Offset(0, 0),
                              blurRadius: 2,
                              color: widget.chatHeadColor.withOpacity(0.5) ?? widget.chatHeadColor ?? Colors.brown,
                            )
                          ],
                        ),
                        child: Center(
                            child: TextRF(
                          UtilRF.getInitials(widget.chatGroup.Name),
                          color: widget.primaryColor ?? Colors.white,
                          size: 25,
                        )),
                      ),
                      widget.hasModifyPermission
                          ? Expanded(
                              child: Center(
                                child: InkWell(
                                  onTap: () {
                                    isMemberAdd ? isMemberAdd = false : isMemberAdd = true;
                                    setState(() {});
                                  },
                                  child: Container(
                                    height: 30,
                                    width: 110,
                                    padding: const EdgeInsets.only(top: 5, bottom: 5, left: 5, right: 5),
                                    decoration: BoxDecoration(
                                      color: Colors.green.withOpacity(0.2),
                                      borderRadius: BorderRadius.all(Radius.circular(10)),
                                    ),
                                    child: Center(
                                        child: TextRF(
                                      isMemberAdd ? 'Group Members' : 'Add Member',
                                      color: Colors.green[600],
                                      size: 13,
                                    )),
                                  ),
                                ),
                              ),
                            )
                          : Expanded(
                              child: SizedBox(),
                            ),
                    ],
                  ),
                  SizedBox(
                    height: 20,
                  ),
                  Center(
                    child: TextRF(
                      widget.chatGroup.Name,
                      size: 18,
                      weight: FontWeight.bold,
                      color: Colors.black,
                      align: TextAlign.center,
                    ),
                  ),
                  SizedBox(
                    height: 20,
                  ),
                  Container(
                    height: 0.3,
                    color: Colors.grey.withOpacity(0.4),
                  ),
                  SizedBox(
                    height: 20,
                  ),
                  isMemberAdd
                      ? Container(
                          height: ScreenRF.height(context) - 300,
                          child: Column(
                            children: [
                              Container(
                                height: 45,
                                margin: const EdgeInsets.all(15),
                                decoration: BoxDecoration(
                                  color: Colors.white,
                                  borderRadius: BorderRadius.all(Radius.circular(50)),
                                  boxShadow: <BoxShadow>[
                                    BoxShadow(
                                      offset: Offset(0, 0),
                                      blurRadius: 3,
                                      color: Colors.black.withOpacity(.2),
                                    ),
                                  ],
                                ),
                                padding: const EdgeInsets.only(left: 12, right: 25, top: 0, bottom: 0),
                                child: TextField(
                                  controller: searchCTRL,
                                  style: TextStyle(fontSize: 19, color: Colors.black),
                                  enabled: true,
                                  keyboardType: TextInputType.text,
                                  autofocus: false,
                                  onChanged: (value) {
                                    searchText = value;
                                    setState(() {});
                                  },
                                  decoration: new InputDecoration(
                                    icon: Icon(Icons.search_sharp),
                                    fillColor: Colors.white,
                                    border: InputBorder.none,
                                    focusColor: Colors.transparent,
                                    filled: true,
                                    hintText: "search",
                                    hintStyle: TextStyle(color: Colors.grey[400], fontWeight: FontWeight.w300),
                                  ),
                                ),
                              ),
                              FutureBuilder(
                                future: getUsersList(searchText),
                                builder: (BuildContext context, AsyncSnapshot<List<ChatUsers>> snapshot) {
                                  if (snapshot.hasData) {
                                    return Container(
                                      child: ListView.builder(
                                        reverse: true,
                                        shrinkWrap: true,
                                        itemCount: snapshot.data.length,
                                        itemBuilder: (context, index) {
                                          ChatUsers chatuser = snapshot.data[index];
                                          return Container(
                                              width: MediaQuery.of(context).size.width - 100,
                                              margin: const EdgeInsets.only(bottom: 10),
                                              decoration: BoxDecoration(
                                                borderRadius: BorderRadius.all(Radius.circular(5)),
                                              ),
                                              child: memberCard(
                                                  userId: chatuser.UserId,
                                                  name: chatuser.UserName,
                                                  isAdd: true,
                                                  userTable: chatuser.UserTable,
                                                  onAdded: () {
                                                    members[chatuser.UserId] = chatuser.UserName;
                                                    initCall();
                                                    setState(() {});
                                                  }));
                                        },
                                      ),
                                    );
                                  } else {
                                    return Text("");
                                  }
                                },
                              ),
                            ],
                          ),
                        )
                      : Container(
                          child: ListView.builder(
                            reverse: true,
                            shrinkWrap: true,
                            itemCount: chatMembers.length,
                            itemBuilder: (context, index) {
                              ChatGroupMember chatMem = chatMembers[index];
                              return Container(
                                  width: MediaQuery.of(context).size.width - 100,
                                  margin: const EdgeInsets.only(bottom: 10),
                                  decoration: BoxDecoration(
                                    color: Colors.white,
                                    borderRadius: BorderRadius.all(Radius.circular(5)),
                                  ),
                                  child: memberCard(
                                      showButton: chatMembers.length > 2 ? true : false,
                                      userId: chatMem.UserId,
                                      name: chatMem.UserName,
                                      userTable: chatMem.UserTable,
                                      onDelete: () {
                                        chatMembers.removeAt(index);
                                        setState(() {});
                                      }));
                            },
                          ),
                        ),
                  SizedBox(
                    height: 20,
                  ),
                ],
              ),
            );
          },
        );
      },
    );
  }

  Future<List<ChatUsers>> getUsersList(String searchText) async {
    var users;
    List<ChatUsers> chatUsers = [];
    if (searchText == null || searchText.isEmpty)
      users = await rf.db.get(ChatUsers(), 'select * from ChatUsers');
    else
      users = await rf.db.get(ChatUsers(), 'select * from ChatUsers where UserName like \'${searchText.trim()}%\'');

    if (users != null) {
      for (ChatUsers row in users) {
        if (!members.containsKey(row.UserId)) chatUsers.add(row);
      }
      return chatUsers;
    } else
      return null;
  }

  void _scrollListener() {
    if (_scrollController.position.extentAfter < 200) {
      if (!isUpdating) {
        print('getting data at :' + _scrollController.position.extentAfter.toString());
        isUpdating = true;
        updateMessages(sendAt: messageList.last.SendAt);
      }
    }
  }
}
